import unittest
import pathlib

from justic.utils import Directories


class TestDirectories(unittest.TestCase):

    def test_items(self):
        directories = Directories()
        self.assertIn('work', directories)
        self.assertIn('build', directories)
        self.assertIn('content', directories)
        self.assertIn('templates', directories)
        self.assertIn('static', directories)

    def test_pathlib(self):
        directories = Directories()
        for item in directories.values():
            self.assertIsInstance(item, pathlib.Path)


if __name__ == '__main__':
    unittest.main()



import sys
import pathlib
import shutil
import logging
import importlib.util

import jinja2


def check_file(file):
    spec = importlib.util.spec_from_file_location('justic.data', file)
    data = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(data)
    content = {name: getattr(data, name) for name in dir(data) if not name.startswith('__')}
    return getattr(data, 'template', None), content


class Directories(dict):
    """Manage the directories"""

    def __init__(self, **kwargs):
        super().__init__()
        self['work'] = pathlib.Path(kwargs.get('workdir', '.')).absolute()
        self['build'] = self['work'] / kwargs.get('build', 'build')
        self['content'] = self['work'] / kwargs.get('content', 'content')
        self['templates'] = self['work'] / kwargs.get('templates', 'templates')
        self['static'] = self['work'] / kwargs.get('static', 'static')


class Justic:

    def __init__(self, **kwargs):
        self.logger = logging.getLogger('justic.Justic')
        self.kwargs = kwargs
        self.directories = Directories(**kwargs)
        self.config = self.directories['work'] / kwargs.get('config', 'justiconf.py')

    def load_global(self):
        if self.config.is_file():
            return check_file(self.config)
        return 'base.html', {}

    def get_outputfile(self, file):
        return self.directories['build'] / file.relative_to(self.directories['content']).with_suffix('.html')

    def render(self):
        tmpenv = jinja2.Environment(loader=jinja2.FileSystemLoader(searchpath=self.directories['templates']))
        default_tempalte, global_content = self.load_global()
        for file in self.directories['content'].glob('**/*.py'):
            template_name, content = check_file(file)
            template = tmpenv.get_template(template_name or default_tempalte)
            content.update(global_content)
            data = template.render(**content)
            self.get_outputfile(file).write_text(data)
            self.logger.info('create file %s', self.get_outputfile(file))
            self.logger.debug('content %s', content)

    def copy_static(self):
        shutil.rmtree(self.directories['build'] / 'static')
        shutil.copytree(self.directories['static'], self.directories['build'] / 'static')

    def run(self):
        self.logger.info('run')
        if self.kwargs.get('delete', False):
            shutil.rmtree(self.directories['build'])
        sys.path.append(str(self.directories['work']))
        self.directories['build'].mkdir(parents=True, exist_ok=True)
        self.copy_static()
        self.render()



from pkg_resources import get_distribution, DistributionNotFound
from justic.utils import Justic


__all__ = ['Justic']

try:
    __version__ = get_distribution('justic').version
except:
    __version__ = 'unknown'
finally:
    del get_distribution, DistributionNotFound



import argparse
import logging

from justic import Justic, __version__


def set_logger(verbose):
    levels = [logging.WARNING, logging.INFO, logging.DEBUG]
    level = levels[min(len(levels) - 1, verbose)]
    logging.basicConfig(level=level,
                        format="%(asctime)s %(levelname)s %(message)s")


def main(argv=None):
    parser = argparse.ArgumentParser(prog='justic')
    parser.add_argument('-V', '--version', action='version', version=__version__)
    parser.add_argument('-v', '--verbose', action='count', default=0)
    parser.add_argument('-d', '--delete', action='store_true')
    parser.add_argument('-c', '--config', default='justiconf.py')
    parser.add_argument('workdir', nargs='?', default='.')

    args = parser.parse_args(argv)
    set_logger(args.verbose)
    justic = Justic(**vars(args))
    justic.run()


if __name__ == '__main__':
    main()



import unittest
import pathlib

from justic.__main__ import main


class TestMain(unittest.TestCase):

    def test_version(self):
        # main(['-V'])
        workdir = pathlib.Path(__file__).parent / 'test_project'
        main([str(workdir)])


if __name__ == '__main__':
    unittest.main()



from setuptools import setup


setup(
    packages=['justic'],
    include_package_data=True,
    use_scm_version=True,
    install_requires=[
        'jinja2',
    ],
    setup_requires=[
        'setuptools_scm',
    ],
    entry_points={
        'console_scripts': [
            'justic=justic.__main__:main',
        ],
    },
)
